@@ -7,9 +7,10 @@ import subprocess
 import os
 import sys
 
-from ipython_genutils.py3compat import which, cast_bytes_py2, getcwd
+import shutil
 from traitlets import Integer, List, Bool, Instance, Unicode, default
 from testpath.tempdir import TemporaryWorkingDirectory
+from typing import Optional
 from .latex import LatexExporter
 
 class LatexFailed(IOError):
@@ -25,7 +26,7 @@ class LatexFailed(IOError):
     
     def __str__(self):
         u = self.__unicode__()
-        return cast_bytes_py2(u)
+        return u
 
 def prepend_to_env_search_path(varname, value, envdict):
     """Add value to the environment variable varname in envdict
@@ -35,16 +36,16 @@ def prepend_to_env_search_path(varname, value, envdict):
     if not value:
         return  # Nothing to add
 
-    envdict[varname] = cast_bytes_py2(value) + os.pathsep + envdict.get(varname, '')
+    envdict[varname] = value + os.pathsep + envdict.get(varname, '')
 
 class PDFExporter(LatexExporter):
     """Writer designed to write to PDF files.
 
-    This inherits from :class:`LatexExporter`. It creates a LaTeX file in
+    This inherits from `LatexExporter`. It creates a LaTeX file in
     a temporary directory using the template machinery, and then runs LaTeX
     to create a pdf.
     """
-    export_from_notebook="pdf"
+    export_from_notebook="PDF via LaTeX"
 
     latex_count = Integer(3,
         help="How many times latex will be called."
@@ -73,9 +74,14 @@ class PDFExporter(LatexExporter):
     def _file_extension_default(self):
         return '.pdf'
 
-    def run_command(self, command_list, filename, count, log_function):
+
+    @default('template_extension')
+    def _template_extension_default(self):
+        return '.tex.j2'
+
+    def run_command(self, command_list, filename, count, log_function, raise_on_failure=None):
         """Run command_list count times.
-        
+
         Parameters
         ----------
         command_list : list
@@ -85,7 +91,10 @@ class PDFExporter(LatexExporter):
             The name of the file to convert.
         count : int
             How many times to run the command.
-        
+        raise_on_failure: Exception class (default None)
+            If provided, will raise the given exception for if an instead of
+            returning False on command failure.
+
         Returns
         -------
         success : bool
@@ -94,22 +103,14 @@ class PDFExporter(LatexExporter):
         """
         command = [c.format(filename=filename) for c in command_list]
 
-        # On windows with python 2.x there is a bug in subprocess.Popen and
-        # unicode commands are not supported
-        if sys.platform == 'win32' and sys.version_info < (3,0):
-            #We must use cp1252 encoding for calling subprocess.Popen
-            #Note that sys.stdin.encoding and encoding.DEFAULT_ENCODING
-            # could be different (cp437 in case of dos console)
-            command = [c.encode('cp1252') for c in command]
-
         # This will throw a clearer error if the command is not found
-        cmd = which(command_list[0])
+        cmd = shutil.which(command_list[0])
         if cmd is None:
             link = "https://nbconvert.readthedocs.io/en/latest/install.html#installing-tex"
             raise OSError("{formatter} not found on PATH, if you have not installed "
                           "{formatter} you may need to do so. Find further instructions "
                           "at {link}.".format(formatter=command_list[0], link=link))
-        
+
         times = 'time' if count == 1 else 'times'
         self.log.info("Running %s %i %s: %s", command_list[0], count, times, command)
         
@@ -136,20 +137,24 @@ class PDFExporter(LatexExporter):
                         out = out.decode('utf-8', 'replace')
                     log_function(command, out)
                     self._captured_output.append(out)
+                    if raise_on_failure:
+                        raise raise_on_failure(
+                            'Failed to run "{command}" command:\n{output}'.format(
+                            command=command, output=out))
                     return False # failure
         return True # success
 
-    def run_latex(self, filename):
+    def run_latex(self, filename, raise_on_failure=LatexFailed):
         """Run xelatex self.latex_count times."""
 
         def log_error(command, out):
             self.log.critical(u"%s failed: %s\n%s", command[0], command, out)
 
         return self.run_command(self.latex_command, filename,
-            self.latex_count, log_error)
+            self.latex_count, log_error, raise_on_failure)
 
-    def run_bib(self, filename):
-        """Run bibtex self.latex_count times."""
+    def run_bib(self, filename, raise_on_failure=False):
+        """Run bibtex one time."""
         filename = os.path.splitext(filename)[0]
 
         def log_error(command, out):
@@ -157,30 +162,28 @@ class PDFExporter(LatexExporter):
                 command[0])
             self.log.debug(u"%s output: %s\n%s", command[0], command, out)
 
-        return self.run_command(self.bib_command, filename, 1, log_error)
+        return self.run_command(self.bib_command, filename, 1, log_error, raise_on_failure)
     
     def from_notebook_node(self, nb, resources=None, **kw):
-        latex, resources = super(PDFExporter, self).from_notebook_node(
+        latex, resources = super().from_notebook_node(
             nb, resources=resources, **kw
         )
         # set texinputs directory, so that local files will be found
         if resources and resources.get('metadata', {}).get('path'):
             self.texinputs = resources['metadata']['path']
         else:
-            self.texinputs = getcwd()
-        
+            self.texinputs = os.getcwd()
+
         self._captured_outputs = []
         with TemporaryWorkingDirectory():
             notebook_name = 'notebook'
             resources['output_extension'] = '.tex'
             tex_file = self.writer.write(latex, resources, notebook_name=notebook_name)
             self.log.info("Building PDF")
-            rc = self.run_latex(tex_file)
-            if rc:
-                rc = self.run_bib(tex_file)
-            if rc:
-                rc = self.run_latex(tex_file)
-            
+            self.run_latex(tex_file)
+            if self.run_bib(tex_file):
+                self.run_latex(tex_file)
+
             pdf_file = notebook_name + '.pdf'
             if not os.path.isfile(pdf_file):
                 raise LatexFailed('\n'.join(self._captured_output))
