@@ -65,6 +65,7 @@ default_filters = {
         'convert_pandoc': filters.convert_pandoc,
         'json_dumps': json.dumps,
         'strip_trailing_newline': filters.strip_trailing_newline,
+        'text_base64': filters.text_base64,
 }
 
 
@@ -273,6 +274,10 @@ class TemplateExporter(Exporter):
         help = "This allows you to exclude output prompts from all templates if set to True."
         ).tag(config=True)
 
+    exclude_output_stdin = Bool(True,
+        help = "This allows you to exclude output of stdin stream from lab template if set to True."
+        ).tag(config=True)
+
     exclude_code_cell = Bool(False,
         help = "This allows you to exclude code cells from all templates if set to True."
         ).tag(config=True)
@@ -375,6 +380,7 @@ class TemplateExporter(Exporter):
                 'include_unknown': not self.exclude_unknown,
                 'include_input': not self.exclude_input,
                 'include_output': not self.exclude_output,
+                'include_output_stdin': not self.exclude_output_stdin,
                 'include_input_prompt': not self.exclude_input_prompt,
                 'include_output_prompt': not self.exclude_output_prompt,
                 'no_prompt': self.exclude_input_prompt and self.exclude_output_prompt,
@@ -499,6 +505,8 @@ class TemplateExporter(Exporter):
                 kwargs = preprocessor.copy()
                 preprocessor_cls = kwargs.pop('type')
                 preprocessor_cls = import_item(preprocessor_cls)
+                if preprocessor_cls.__name__ in self.config:
+                    kwargs.update(self.config[preprocessor_cls.__name__])
                 preprocessor = preprocessor_cls(**kwargs)
                 self.register_preprocessor(preprocessor)
 
@@ -546,7 +554,7 @@ class TemplateExporter(Exporter):
             except OSError:
                 pass
 
-        return self.extra_template_paths + additional_paths + paths
+        return paths + self.extra_template_paths + additional_paths
 
     @classmethod
     def get_compatibility_base_template_conf(cls, name):
