@@ -12,6 +12,7 @@ import os
 import re
 import textwrap
 import warnings
+import base64
 
 from urllib.parse import quote
 
@@ -35,6 +36,7 @@ __all__ = [
     'ascii_only',
     'prevent_list_blocks',
     'strip_trailing_newline',
+    'text_base64',
 ]
 
 
@@ -83,7 +85,10 @@ def _convert_header_id(header_contents):
 
     For use on markdown headings.
     """
-    return header_contents.replace(' ', '-')
+    # Valid IDs need to be non-empty and contain no space characters, but are otherwise arbitrary.
+    # However, these IDs are also used in URL fragments, which are more restrictive, so we URL
+    # encode any characters that are not valid in URL fragments.
+    return quote(header_contents.replace(' ', '-'), safe="?/:@!$&'()*+,;=")
 
 
 def add_anchor(html, anchor_link_text=u'¶'):
@@ -246,3 +251,9 @@ def strip_trailing_newline(text):
     if text.endswith('\n'):
         text = text[:-1]
     return text
+
+def text_base64(text):
+    """
+    Encode base64 text
+    """
+    return base64.b64encode(text.encode()).decode()
