@@ -13,6 +13,9 @@ import logging
 import sys
 import os
 import glob
+import asyncio
+from textwrap import fill, dedent
+from ipython_genutils.text import indent
 
 from jupyter_core.application import JupyterApp, base_aliases, base_flags
 from traitlets.config import catch_config_error, Configurable
@@ -41,7 +44,7 @@ class DottedOrNone(DottedObjectName):
 
     def validate(self, obj, value):
         if value is not None and len(value) > 0:
-            return super(DottedOrNone, self).validate(obj, value)
+            return super().validate(obj, value)
         else:
             return value
             
@@ -49,7 +52,8 @@ nbconvert_aliases = {}
 nbconvert_aliases.update(base_aliases)
 nbconvert_aliases.update({
     'to' : 'NbConvertApp.export_format',
-    'template' : 'TemplateExporter.template_file',
+    'template' : 'TemplateExporter.template_name',
+    'template-file' : 'TemplateExporter.template_file',
     'writer' : 'NbConvertApp.writer_class',
     'post': 'NbConvertApp.postprocessor_class',
     'output': 'NbConvertApp.output_base',
@@ -118,11 +122,19 @@ nbconvert_flags.update({
         {'TemplateExporter' : {
             'exclude_output_prompt' : True,
             'exclude_input': True,
+            'exclude_input_prompt': True,
             }
         },
         """Exclude input cells and output prompts from converted document. 
         This mode is ideal for generating code-free reports."""
         ),
+    'allow-chromium-download' : (
+        {'WebPDFExporter' : {
+            'allow_chromium_download' : True,
+            }
+        },
+        """Whether to allow downloading chromium if no suitable version is found on the system."""
+        ),
 })
 
 
@@ -170,17 +182,14 @@ class NbConvertApp(JupyterApp):
     output_files_dir = Unicode('{notebook_name}_files',
          help='''Directory to copy extra files (figures) to.
                '{notebook_name}' in the string will be converted to notebook
-               basename'''
+               basename.'''
     ).tag(config=True)
 
     examples = Unicode(u"""
         The simplest way to use nbconvert is
         
-        > jupyter nbconvert mynotebook.ipynb
-        
-        which will convert mynotebook.ipynb to the default format (probably HTML).
-        
-        You can specify the export format with `--to`.
+        > jupyter nbconvert mynotebook.ipynb --to html
+
         Options include {formats}.
         
         > jupyter nbconvert --to latex mynotebook.ipynb
@@ -189,7 +198,7 @@ class NbConvertApp(JupyterApp):
         'base', 'article' and 'report'.  HTML includes 'basic' and 'full'. You
         can specify the flavor of the format used.
 
-        > jupyter nbconvert --to html --template basic mynotebook.ipynb
+        > jupyter nbconvert --to html --template lab mynotebook.ipynb
         
         You can also pipe the output to stdout, rather than a file
         
@@ -255,17 +264,12 @@ class NbConvertApp(JupyterApp):
         if new:
             self.postprocessor_factory = import_item(new)
 
-    ipywidgets_base_url = Unicode("https://unpkg.com/",
-                                  help="URL base for ipywidgets package").tag(config=True)
-
-
     export_format = Unicode(
-        'html',
         allow_none=False,
         help="""The export format to be used, either one of the built-in formats
         {formats}
         or a dotted object name that represents the import path for an
-        `Exporter` class""".format(formats=get_export_names())
+        ``Exporter`` class""".format(formats=get_export_names())
     ).tag(config=True)
 
     notebooks = List([], help="""List of notebooks to convert.
@@ -278,8 +282,12 @@ class NbConvertApp(JupyterApp):
     @catch_config_error
     def initialize(self, argv=None):
         """Initialize application, notebooks, writer, and postprocessor"""
+        # See https://bugs.python.org/issue37373 :(
+        if sys.version_info[0] == 3 and sys.version_info[1] >= 8 and sys.platform.startswith('win'):
+            asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
+
         self.init_syspath()
-        super(NbConvertApp, self).initialize(argv)
+        super().initialize(argv)
         self.init_notebooks()
         self.init_writer()
         self.init_postprocessor()
@@ -334,7 +342,7 @@ class NbConvertApp(JupyterApp):
 
     def start(self):
         """Run start after initialization process has completed"""
-        super(NbConvertApp, self).start()
+        super().start()
         self.convert_notebooks()
 
     def init_single_notebook_resources(self, notebook_filename):
@@ -344,7 +352,6 @@ class NbConvertApp(JupyterApp):
 
         Returns
         -------
-
         dict
             resources dictionary for a single notebook that MUST include the following keys:
                 - config_dir: the location of the Jupyter config directory
@@ -374,8 +381,6 @@ class NbConvertApp(JupyterApp):
 
         resources['output_files_dir'] = output_files_dir
 
-        resources['ipywidgets_base_url'] = self.ipywidgets_base_url
-
         return resources
 
     def export_single_notebook(self, notebook_filename, resources, input_buffer=None):
@@ -397,7 +402,6 @@ class NbConvertApp(JupyterApp):
         Returns
         -------
         output
-
         dict
             resources (possibly modified)
         """
@@ -492,16 +496,22 @@ class NbConvertApp(JupyterApp):
                 """
             )
             self.exit(1)
-        
-        # initialize the exporter
-        cls = get_exporter(self.export_format)
-        self.exporter = cls(config=self.config)
 
         # no notebooks to convert!
         if len(self.notebooks) == 0 and not self.from_stdin:
             self.print_help()
             sys.exit(-1)
 
+        if not self.export_format:
+            raise ValueError(
+                "Please specify an output format with '--to <format>'."
+                f"\nThe following formats are available: {get_export_names()}"
+            )
+
+        # initialize the exporter
+        cls = get_exporter(self.export_format)
+        self.exporter = cls(config=self.config)
+
         # convert each notebook
         if not self.from_stdin:
             for notebook_filename in self.notebooks:
@@ -510,6 +520,50 @@ class NbConvertApp(JupyterApp):
             input_buffer = unicode_stdin_stream()
             # default name when conversion from stdin
             self.convert_single_notebook("notebook.ipynb", input_buffer=input_buffer)
+
+    def document_flag_help(self):
+        """
+        Return a string containing descriptions of all the flags.
+        """
+        flags = "The following flags are defined:\n\n"
+        for flag, (cfg, fhelp) in self.flags.items():
+            flags += "{}\n".format(flag)
+            flags += indent(fill(fhelp, 80)) + '\n\n'
+            flags += indent(fill("Long Form: "+str(cfg), 80)) + '\n\n'
+        return flags
+
+    def document_alias_help(self):
+        """Return a string containing all of the aliases"""
+
+        aliases = "The folowing aliases are defined:\n\n"
+        for alias, longname in self.aliases.items():
+            aliases += "\t**{}** ({})\n\n".format(alias, longname)
+        return aliases
+
+    def document_config_options(self):
+        """
+        Provides a much improves version of the configuration documentation by
+        breaking the configuration options into app, exporter, writer,
+        preprocessor, postprocessor, and other sections.
+        """
+        categories = {category: [c for c in self._classes_inc_parents() if category in c.__name__.lower()]
+                    for category in ['app', 'exporter', 'writer', 'preprocessor', 'postprocessor']}
+        accounted_for = {c for category in categories.values() for c in category}
+        categories['other']=  [c for c in self._classes_inc_parents() if c not in accounted_for]
+
+        header = dedent("""
+                        {section} Options
+                        -----------------------
+
+                        """)
+        sections = ""
+        for category in categories:
+            sections += header.format(section=category.title())
+            if category in ['exporter','preprocessor','writer']:
+                sections += ".. image:: _static/{image}_inheritance.png\n\n".format(image=category)
+            sections += '\n'.join(c.class_config_rst_doc() for c in categories[category])
+
+        return sections.replace(' : ',r' \: ')
             
 #-----------------------------------------------------------------------------
 # Main entry point
